use crate::drivers::vga::{Vga13h, VgaColor};

use super::{falling::FallingTetromino, TetrisBoard, TetrisCell};

impl TetrisBoard {
	fn draw_cell(&self, fb: &mut [u8], color: VgaColor, x: usize, y: usize) {
		let color = color.as_u8();
		for yo in 0..TetrisBoard::CELL_SIZE {
			let left = Vga13h::pix_idx(x, y + yo);
			let right = Vga13h::pix_idx(x + TetrisBoard::CELL_SIZE, y + yo);
			fb[left..right].copy_from_slice(&[color; TetrisBoard::CELL_SIZE]);
		}
	}

	fn draw_frame(&self, fb: &mut [u8], x: usize, y: usize) {
		let color = VgaColor::Gray.as_u8();
		for yo in 0..TetrisBoard::CELL_SIZE {
			let left = Vga13h::pix_idx(x, y + yo);
			let right = Vga13h::pix_idx(x + TetrisBoard::CELL_SIZE, y + yo);
			fb[left..right].copy_from_slice(&[color; TetrisBoard::CELL_SIZE]);
		}
	}

	pub fn draw(&self, vga: &mut Vga13h, falling: Option<&FallingTetromino>) {
		let fb = vga.get_fb();

		// Draw cells
		for bx in 0..TetrisBoard::BOARD_WIDTH {
			for by in 0..TetrisBoard::BOARD_HEIGHT {
				let cell = self.board[by * TetrisBoard::BOARD_WIDTH + bx];
				let dx = (bx + 1) * TetrisBoard::CELL_SIZE;
				let dy = (by + 1) * TetrisBoard::CELL_SIZE;

				if let TetrisCell::Filled { color } = cell {
					self.draw_cell(fb, color, dx, dy);
				} else {
					self.draw_cell(fb, VgaColor::Black, dx, dy);
				}
			}
		}

		// Draw falling tetromino
		if let Some(falling) = falling {
			for (x, y) in falling.tiles() {
				let dx = (x + 1) * TetrisBoard::CELL_SIZE;
				let dy = (y + 1) * TetrisBoard::CELL_SIZE;

				self.draw_cell(fb, falling.color, dx, dy);
			}
		}

		// Draw frame
		for bx in 0..TetrisBoard::BOARD_WIDTH + 2 {
			self.draw_frame(fb, bx * TetrisBoard::CELL_SIZE, 0);
			self.draw_frame(
				fb,
				bx * TetrisBoard::CELL_SIZE,
				(TetrisBoard::BOARD_HEIGHT + 1) * TetrisBoard::CELL_SIZE,
			);
		}
		for by in 0..TetrisBoard::BOARD_HEIGHT + 2 {
			self.draw_frame(fb, 0, by * 9);
			self.draw_frame(
				fb,
				(TetrisBoard::BOARD_WIDTH + 1) * TetrisBoard::CELL_SIZE,
				by * TetrisBoard::CELL_SIZE,
			);
		}
	}
}
